@@ -4,10 +4,12 @@ import datetime as dt
 import json
 import logging as lg
 import re
+import socket
 import time
 from collections import OrderedDict
 from hashlib import sha1
 from pathlib import Path
+from urllib.parse import urlparse
 
 import numpy as np
 import requests
@@ -19,6 +21,9 @@ from . import utils
 from . import utils_geo
 from ._errors import CacheOnlyModeInterrupt
 
+# capture getaddrinfo function to use original later after mutating it
+_original_getaddrinfo = socket.getaddrinfo
+
 
 def _get_osm_filter(network_type):
     """
@@ -245,15 +250,95 @@ def _get_http_headers(user_agent=None, referer=None, accept_language=None):
     return headers
 
 
-def _get_pause(recursive_delay=5, default_duration=60):
+def _get_host_by_name(host):
+    """
+    Resolve IP address from host using Google's public API for DNS over HTTPS.
+
+    Necessary fallback as socket.gethostbyname will not always work when using
+    a proxy. See https://developers.google.com/speed/public-dns/docs/doh/json
+
+    Parameters
+    ----------
+    host : string
+        the host to consistently resolve the IP address of
+
+    Returns
+    -------
+    ip_address : string
+        resolved IP address
+    """
+    dns_url = f"https://dns.google/resolve?name={host}"
+    response = requests.get(dns_url)
+    data = response.json()
+
+    # status = 0 means NOERROR: standard DNS response code
+    if response.ok and data["Status"] == 0:
+        ip_address = data["Answer"][0]["data"]
+        utils.log(f"Google resolved '{host}' to '{ip_address}'")
+        return ip_address
+
+    # in case host could not be resolved return the host itself
+    else:
+        utils.log(f"Google could not resolve '{host}'. Response status: {data['Status']}")
+        return host
+
+
+def _config_dns(url):
+    """
+    Force socket.getaddrinfo to use IP address instead of host.
+
+    Resolves the URL's domain to an IP address so that we use the same server
+    for both 1) checking the necessary pause duration and 2) sending the query
+    itself even if there is round-robin redirecting among multiple server
+    machines on the server-side. Mutates the getaddrinfo function so it uses
+    the same IP address everytime it finds the host name in the URL.
+
+    For example, the domain overpass-api.de just redirects to one of its
+    subdomains (currently z.overpass-api.de and lz4.overpass-api.de). So if we
+    check the status endpoint of overpass-api.de, we may see results for
+    subdomain z, but when we submit the query itself it gets redirected to
+    subdomain lz4. This could result in violating server lz4's slot management
+    timing.
+
+    Parameters
+    ----------
+    url : string
+        the URL to consistently resolve the IP address of
+
+    Returns
+    -------
+    None
+    """
+    host = urlparse(url).netloc.split(":")[0]
+    try:
+        ip = socket.gethostbyname(host)
+    except socket.gaierror:  # pragma: no cover
+        # this error occurs sometimes when using a proxy. instead, you must
+        # get IP address using google's public JSON API for DNS over HTTPS
+        ip = _get_host_by_name(host)[0]
+
+    def _getaddrinfo(*args):
+        if args[0] == host:
+            utils.log(f"Resolved {host} to {ip}")
+            return _original_getaddrinfo(ip, *args[1:])
+        else:
+            return _original_getaddrinfo(*args)
+
+    socket.getaddrinfo = _getaddrinfo
+
+
+def _get_pause(base_endpoint, recursive_delay=5, default_duration=60):
     """
     Get a pause duration from the Overpass API status endpoint.
 
     Check the Overpass API status endpoint to determine how long to wait until
-    next slot is available.
+    the next slot is available. You can disable this via the `settings`
+    module's `overpass_rate_limit` setting.
 
     Parameters
     ----------
+    base_endpoint : string
+        base Overpass API endpoint (without "/status" at the end)
     recursive_delay : int
         how long to wait between recursive calls if the server is currently
         running a query
@@ -264,17 +349,23 @@ def _get_pause(recursive_delay=5, default_duration=60):
     -------
     pause : int
     """
+    if not settings.overpass_rate_limit:  # pragma: no cover
+        # if overpass rate limiting is False, then there is zero pause
+        return 0
+
+    sc = None
+
     try:
-        url = settings.overpass_endpoint.rstrip("/") + "/status"
-        response = requests.get(url, headers=_get_http_headers())
+        url = base_endpoint.rstrip("/") + "/status"
+        response = requests.get(url, headers=_get_http_headers(), **settings.requests_kwargs)
+        sc = response.status_code
         status = response.text.split("\n")[3]
         status_first_token = status.split(" ")[0]
 
     except Exception:  # pragma: no cover
         # if we cannot reach the status endpoint or parse its output, log an
         # error and return default duration
-        sc = response.status_code
-        utils.log(f"Unable to query {url} got status {sc}", level=lg.ERROR)
+        utils.log(f"Unable to query {url}, got status {sc}", level=lg.ERROR)
         return default_duration
 
     try:
@@ -295,7 +386,7 @@ def _get_pause(recursive_delay=5, default_duration=60):
         # check back in recursive_delay seconds
         elif status_first_token == "Currently":
             time.sleep(recursive_delay)
-            pause = _get_pause()
+            pause = _get_pause(base_endpoint)
 
         # any other status is unrecognized: log error, return default duration
         else:
@@ -576,8 +667,12 @@ def nominatim_request(params, request_type="search", pause=1, error_pause=60):
     if request_type not in {"search", "reverse", "lookup"}:  # pragma: no cover
         raise ValueError('Nominatim request_type must be "search", "reverse", or "lookup"')
 
+    # resolve url to same IP even if there is server round-robin redirecting
+    _config_dns(settings.nominatim_endpoint.rstrip("/"))
+
     # prepare Nominatim API URL and see if request already exists in cache
     url = settings.nominatim_endpoint.rstrip("/") + "/" + request_type
+
     prepared_url = requests.Request("GET", url, params=params).prepare().url
     cached_response_json = _retrieve_from_cache(prepared_url)
 
@@ -596,7 +691,13 @@ def nominatim_request(params, request_type="search", pause=1, error_pause=60):
         # transmit the HTTP GET request
         utils.log(f"Get {prepared_url} with timeout={settings.timeout}")
         headers = _get_http_headers()
-        response = requests.get(url, params=params, timeout=settings.timeout, headers=headers)
+        response = requests.get(
+            url,
+            params=params,
+            timeout=settings.timeout,
+            headers=headers,
+            **settings.requests_kwargs,
+        )
         sc = response.status_code
 
         # log the response size and domain
@@ -644,9 +745,14 @@ def overpass_request(data, pause=None, error_pause=60):
     -------
     response_json : dict
     """
+    base_endpoint = settings.overpass_endpoint
+
+    # resolve url to same IP even if there is server round-robin redirecting
+    _config_dns(base_endpoint)
+
     # define the Overpass API URL, then construct a GET-style URL as a string to
     # hash to look up/save to cache
-    url = settings.overpass_endpoint.rstrip("/") + "/interpreter"
+    url = base_endpoint.rstrip("/") + "/interpreter"
     prepared_url = requests.Request("GET", url, params=data).prepare().url
     cached_response_json = _retrieve_from_cache(prepared_url, check_remark=True)
 
@@ -657,14 +763,16 @@ def overpass_request(data, pause=None, error_pause=60):
     else:
         # if this URL is not already in the cache, pause, then request it
         if pause is None:
-            this_pause = _get_pause()
+            this_pause = _get_pause(base_endpoint)
         utils.log(f"Pausing {this_pause} seconds before making HTTP POST request")
         time.sleep(this_pause)
 
         # transmit the HTTP POST request
         utils.log(f"Post {prepared_url} with timeout={settings.timeout}")
         headers = _get_http_headers()
-        response = requests.post(url, data=data, timeout=settings.timeout, headers=headers)
+        response = requests.post(
+            url, data=data, timeout=settings.timeout, headers=headers, **settings.requests_kwargs
+        )
         sc = response.status_code
 
         # log the response size and domain
@@ -682,7 +790,7 @@ def overpass_request(data, pause=None, error_pause=60):
                 # 429 is 'too many requests' and 504 is 'gateway timeout' from
                 # server overload: handle these by pausing then recursively
                 # re-trying until we get a valid response from the server
-                this_pause = error_pause + _get_pause()
+                this_pause = error_pause + _get_pause(base_endpoint)
                 utils.log(f"{domain} returned {sc}: retry in {this_pause} secs", level=lg.WARNING)
                 time.sleep(this_pause)
                 response_json = overpass_request(data, pause, error_pause)
