@@ -4,12 +4,12 @@ import logging as lg
 
 import geopandas as gpd
 import networkx as nx
-import pandas as pd
 from shapely.geometry import LineString
+from shapely.geometry import MultiPolygon
 from shapely.geometry import Point
 from shapely.geometry import Polygon
-from shapely.ops import unary_union
 
+from . import stats
 from . import utils
 from . import utils_graph
 
@@ -222,11 +222,14 @@ def simplify_graph(G, strict=True, remove_rings=True):
         topologically simplified graph, with a new `geometry` attribute on
         each simplified edge
     """
-    if "simplified" in G.graph and G.graph["simplified"]:
+    if "simplified" in G.graph and G.graph["simplified"]:  # pragma: no cover
         raise Exception("This graph has already been simplified, cannot simplify it again.")
 
     utils.log("Begin topologically simplifying the graph...")
 
+    # define edge segment attributes to sum upon edge simplification
+    attrs_to_sum = {"length", "travel_time"}
+
     # make a copy to not mutate original graph object caller passed in
     G = G.copy()
     initial_node_count = len(G)
@@ -239,48 +242,51 @@ def simplify_graph(G, strict=True, remove_rings=True):
 
         # add the interstitial edges we're removing to a list so we can retain
         # their spatial geometry
-        edge_attributes = dict()
+        path_attributes = dict()
         for u, v in zip(path[:-1], path[1:]):
 
             # there should rarely be multiple edges between interstitial nodes
             # usually happens if OSM has duplicate ways digitized for just one
             # street... we will keep only one of the edges (see below)
-            if G.number_of_edges(u, v) != 1:
-                utils.log(f"Found multiple edges between {u} and {v} when simplifying")
+            edge_count = G.number_of_edges(u, v)
+            if edge_count != 1:
+                utils.log(f"Found {edge_count} edges between {u} and {v} when simplifying")
 
             # get edge between these nodes: if multiple edges exist between
             # them (see above), we retain only one in the simplified graph
-            edge = G.edges[u, v, 0]
-            for key in edge:
-                if key in edge_attributes:
+            edge_data = G.edges[u, v, 0]
+            for attr in edge_data:
+                if attr in path_attributes:
                     # if this key already exists in the dict, append it to the
                     # value list
-                    edge_attributes[key].append(edge[key])
+                    path_attributes[attr].append(edge_data[attr])
                 else:
                     # if this key doesn't already exist, set the value to a list
                     # containing the one value
-                    edge_attributes[key] = [edge[key]]
+                    path_attributes[attr] = [edge_data[attr]]
 
-        for key in edge_attributes:
-            # don't touch the length attribute, we'll sum it at the end
-            if len(set(edge_attributes[key])) == 1 and key != "length":
+        # consolidate the path's edge segments' attribute values
+        for attr in path_attributes:
+            if attr in attrs_to_sum:
+                # if this attribute must be summed, sum it now
+                path_attributes[attr] = sum(path_attributes[attr])
+            elif len(set(path_attributes[attr])) == 1:
                 # if there's only 1 unique value in this attribute list,
-                # consolidate it to the single value (the zero-th)
-                edge_attributes[key] = edge_attributes[key][0]
-            elif key != "length":
-                # otherwise, if there are multiple values, keep one of each value
-                edge_attributes[key] = list(set(edge_attributes[key]))
+                # consolidate it to the single value (the zero-th):
+                path_attributes[attr] = path_attributes[attr][0]
+            else:
+                # otherwise, if there are multiple values, keep one of each
+                path_attributes[attr] = list(set(path_attributes[attr]))
 
-        # construct the geometry and sum the lengths of the segments
-        edge_attributes["geometry"] = LineString(
+        # construct the new consolidated edge's geometry for this path
+        path_attributes["geometry"] = LineString(
             [Point((G.nodes[node]["x"], G.nodes[node]["y"])) for node in path]
         )
-        edge_attributes["length"] = sum(edge_attributes["length"])
 
-        # add the nodes and edges to their lists for processing at the end
+        # add the nodes and edge to their lists for processing at the end
         all_nodes_to_remove.extend(path[1:-1])
         all_edges_to_add.append(
-            {"origin": path[0], "destination": path[-1], "attr_dict": edge_attributes}
+            {"origin": path[0], "destination": path[-1], "attr_dict": path_attributes}
         )
 
     # for each edge to add in the list we assembled, create a new edge between
@@ -330,8 +336,9 @@ def consolidate_intersections(
     it uses a topological (and slower but more accurate) algorithm to identify
     "topologically close" nodes, merge them, then rebuild/return the graph.
     Returned graph's node IDs represent clusters rather than osmids. Refer to
-    nodes' osmid attributes for original osmids. If multiple nodes were merged
-    together, the osmid attribute is a list of merged nodes' osmids.
+    nodes' osmid_original attributes for original osmids. If multiple nodes
+    were merged together, the osmid_original attribute is a list of merged
+    nodes' osmids.
 
     Divided roads are often represented by separate centerline edges. The
     intersection of two divided roads thus creates 4 nodes, representing where
@@ -373,9 +380,7 @@ def consolidate_intersections(
     """
     # if dead_ends is False, discard dead-ends to retain only intersections
     if not dead_ends:
-        spn = nx.get_node_attributes(G, "street_count")
-        if set(spn) != set(G.nodes):
-            utils.log("Graph nodes changed since `street_count`s were calculated", level=lg.WARN)
+        spn = stats.streets_per_node(G)
         dead_end_nodes = [node for node, count in spn.items() if count <= 1]
 
         # make a copy to not mutate original graph object caller passed in
@@ -398,64 +403,30 @@ def consolidate_intersections(
             return _merge_nodes_geometric(G, tolerance).centroid
 
 
-def _merge_nodes_geometric(G, tolerance, chunk=True):
+def _merge_nodes_geometric(G, tolerance):
     """
     Geometrically merge nodes within some distance of each other.
 
-    If chunk=True, it sorts the nodes GeoSeries by geometry x and y values (to
-    make unary_union faster), then buffers by tolerance. Next it divides the
-    nodes GeoSeries into n-sized chunks, where n = the square root of the
-    number of nodes. Then it runs unary_union on each chunk, and then runs
-    unary_union on the resulting unary unions. This is much faster on large
-    graphs (n>100000) because of how unary_union's runtime scales with vertex
-    count. But chunk=False is usually faster on small and medium sized graphs.
-    This hacky method will hopefully be made obsolete when shapely becomes
-    vectorized by incorporating the pygeos codebase.
-
     Parameters
     ----------
     G : networkx.MultiDiGraph
         a projected graph
     tolerance : float
-        nodes are buffered to this distance (in graph's geometry's units) and
-        subsequent overlaps are dissolved into a single polygon
-    chunk : bool
-        if True, divide nodes into geometrically sorted chunks to improve the
-        speed of unary_union operation by running it on each chunk and then
-        running it on the results of those runs
+        buffer nodes to this distance (in graph's geometry's units) then merge
+        overlapping polygons into a single polygon via a unary union operation
 
     Returns
     -------
-    merged_nodes : GeoSeries
+    merged : GeoSeries
         the merged overlapping polygons of the buffered nodes
     """
-    # yield n-sized chunks of list one at a time
-    def get_chunks(lst, n):
-        for i in range(0, len(lst), n):
-            yield lst[i : i + n]
-
-    gs_nodes = utils_graph.graph_to_gdfs(G, edges=False)["geometry"]
-
-    if not chunk:
-        # buffer nodes GeoSeries then get unary union to merge overlaps
-        merged_nodes = gs_nodes.buffer(tolerance).unary_union
-
-    else:
-        # sort nodes GeoSeries by geometry x/y vals to make unary_union faster
-        idx = pd.DataFrame((gs_nodes.x, gs_nodes.y)).T.sort_values([0, 1]).index
-
-        # buffer nodes by tolerance then generate n-sized chunks of nodes
-        n = int(len(G) ** 0.5)
-        chunks = get_chunks(gs_nodes.loc[idx].buffer(tolerance).values, n)
-
-        # unary_union each chunk then unary_union those unary unions
-        merged_nodes = unary_union([unary_union(chunk) for chunk in chunks])
+    # buffer nodes GeoSeries then get unary union to merge overlaps
+    merged = utils_graph.graph_to_gdfs(G, edges=False)["geometry"].buffer(tolerance).unary_union
 
     # if only a single node results, make it iterable to convert to GeoSeries
-    if isinstance(merged_nodes, Polygon):
-        merged_nodes = [merged_nodes]
+    merged = MultiPolygon([merged]) if isinstance(merged, Polygon) else merged
 
-    return gpd.GeoSeries(list(merged_nodes), crs=G.graph["crs"])
+    return gpd.GeoSeries(merged.geoms, crs=G.graph["crs"])
 
 
 def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=True):
@@ -470,8 +441,9 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
     a projected graph to work in meaningful and consistent units like meters.
 
     Returned graph's node IDs represent clusters rather than osmids. Refer to
-    nodes' osmid attributes for original osmids. If multiple nodes were merged
-    together, the osmid attribute is a list of merged nodes' osmids.
+    nodes' osmid_original attributes for original osmids. If multiple nodes
+    were merged together, the osmid_original attribute is a list of merged
+    nodes' osmids.
 
     Parameters
     ----------
@@ -506,7 +478,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
     # graph's node points then spatial join to give each node the label of
     # cluster it's within
     node_points = utils_graph.graph_to_gdfs(G, edges=False)[["geometry"]]
-    gdf = gpd.sjoin(node_points, node_clusters, how="left", op="within")
+    gdf = gpd.sjoin(node_points, node_clusters, how="left", predicate="within")
     gdf = gdf.drop(columns="geometry").rename(columns={"index_right": "cluster"})
 
     # STEP 3
@@ -531,6 +503,9 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
                     gdf.loc[wcc, "cluster"] = f"{cluster_label}-{suffix}"
                     suffix += 1
 
+    # give nodes unique integer IDs (subclusters with suffixes are strings)
+    gdf["cluster"] = gdf["cluster"].factorize()[0]
+
     # STEP 4
     # create new empty graph and copy over misc graph data
     H = nx.MultiDiGraph()
@@ -545,17 +520,23 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
         osmids = nodes_subset.index.to_list()
         if len(osmids) == 1:
             # if cluster is a single node, add that node to new graph
-            H.add_node(cluster_label, **G.nodes[osmids[0]])
+            osmid = osmids[0]
+            H.add_node(cluster_label, osmid_original=osmid, **G.nodes[osmid])
         else:
             # if cluster is multiple merged nodes, create one new node to
             # represent them
             H.add_node(
                 cluster_label,
-                osmid=str(osmids),
+                osmid_original=str(osmids),
                 x=nodes_subset["x"].iloc[0],
                 y=nodes_subset["y"].iloc[0],
             )
 
+    # calculate street_count attribute for all nodes lacking it
+    null_nodes = [n for n, sc in H.nodes(data="street_count") if sc is None]
+    street_count = stats.count_streets_per_node(H, nodes=null_nodes)
+    nx.set_node_attributes(H, street_count, name="street_count")
+
     if not G.edges or not reconnect_edges:
         # if reconnect_edges is False or there are no edges in original graph
         # (after dead-end removed), then skip edges and return new graph as-is
@@ -592,7 +573,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
             y = H.nodes[cluster_label]["y"]
             xy = [(x, y)]
 
-            # for each edge incident to this new merged node, update its
+            # for each edge incident on this new merged node, update its
             # geometry to extend to/from the new node's point coords
             in_edges = set(H.in_edges(cluster_label, keys=True))
             out_edges = set(H.out_edges(cluster_label, keys=True))
